/*
  ==================================================================================================
  ___ _  _ _  __
 |_ _| \| | |/ /
  | || .` | ' < 
 |___|_|\_|_|\_\
                
  DialogCtrlInk maintains state and provides functionality for inserting ink    
  ==================================================================================================
  Inserting Ink
    isDrawing - whether the dialog is currently inserting ink
    drawData - the data specifying the lines to be drawn
*/
var DialogCtrlInk = function() {
  'use strict';
  
  var self;
  var ui;
  var nbInterface;
  var dialogCtrl;

  function DialogCtrlInk(pUi, pNbInterface, pDialogCtrl) {
    self = this;
    ui = pUi;
    nbInterface = pNbInterface;
    dialogCtrl = pDialogCtrl;
    this.isDrawing = false;
    this.drawData = undefined;
    this.color = {R: 0, G: 0, B: 0}; //which color is currently selected
  }

  DialogCtrlInk.prototype.insertToPage = function() {
    self.prepInkToPage(false);
  };

  DialogCtrlInk.prototype.insertToNewPage = function () {
    self.prepInkToPage(true);
  };

  DialogCtrlInk.prototype.activateColorBlack = function($clickedButton) {
    ui.inkButtonClick($clickedButton);
    this.setColor(0, 0, 0);
  };

  DialogCtrlInk.prototype.activateColorRed = function($clickedButton) {
    ui.inkButtonClick($clickedButton);
    this.setColor(255, 0, 0);
  };

  DialogCtrlInk.prototype.activateColorBlue = function($clickedButton) {
    ui.inkButtonClick($clickedButton);
    this.setColor(0, 0, 255);
  };

  DialogCtrlInk.prototype.activateColorGreen = function($clickedButton) {
    ui.inkButtonClick($clickedButton);
    this.setColor(0, 128, 0);
  };

  DialogCtrlInk.prototype.setColor = function(r, g, b) {
    this.color.R = r;
    this.color.G = g;
    this.color.B = b;
  };

  // Tells the add-on to draw ink
  DialogCtrlInk.prototype.prepInkToPage = function(toNewPage) {
    if (!this.isDrawing) {
      this.isDrawing = true;
      printDebug("Dialog = dialogDraw");
      this.drawData = this.constructData(toNewPage);
      ui.status.toggleMessage(true, ui.MESSAGE_WAIT, 2000);
      nbInterface.postEvent("dialogStartingDraw");
    }
  };

  // Prepares drawing information to send to the add-on
  DialogCtrlInk.prototype.constructData = function(toNewPage) {
    var data = {};
    data.messageIndex = dialogCtrl.board.messageIndex;
    data.toNewPage = toNewPage;
    var pageSize = nbInterface.getPageVisibleArea();
    var aspectRatio = BoardState.session.boardWidth / BoardState.session.boardHeight;
    if (!toNewPage) {
      if (pageSize.x < 0) {
        data.inkX = 0;
        pageSize.width += (2 * pageSize.x);
        pageSize.x = 0;
      } else {
        data.inkX = pageSize.x;
      }
      data.inkY = pageSize.y;
      if ((pageSize.width / pageSize.height) > aspectRatio) {
        data.inkWidth = aspectRatio * pageSize.height;
      } else {
        data.inkWidth = pageSize.width;
      }
    } else {
      var newPage = nbInterface.addPage();
      data.page = newPage;
    }
    if (dialogCtrl.crop.isCropping) {
      var horizScale = ui.elems.$canvas.width() / BoardState.session.boardWidth;
      var vertScale = ui.elems.$canvas.height() / BoardState.session.boardHeight;
      var crop = {};
      crop.width = ui.elems.$cropperBox.width() / horizScale;
      crop.height = ui.elems.$cropperBox.height() / vertScale;
      crop.x = ui.elems.$cropperBox.position().left / horizScale;
      crop.y = ui.elems.$cropperBox.position().top / vertScale;
      data.crop = crop;
    }
    return data;
  };

  return DialogCtrlInk;
}();