/*
  ==================================================================================================
  _    _ _____ 
 | |  | |_   _|
 | |  | | | |  
 | |  | | | |  
 | |__| |_| |_ 
  \____/|_____|
               
  
  Holds all the jQuery DOM elements, changes them as needed 
  ==================================================================================================
*/

var UI = function() {
  'use strict';
  
  var self;

  function UI() {
    // Rename to put the objects in scope of UI's functions
    self = this;
    // Set this again for scoping issues
    //  MESSAGE_LOADING - displayed at the initial message fetching
    //  MESSAGE_WAIT - displayed when ink is being drawn to the page
    //  MESSAGE_EMPTY - display at an empty session
    this.MESSAGE_LOADING = NB.localize("Loading board state, please wait...", "Loading a board placeholder text");
    this.MESSAGE_WAIT = NB.localize("Drawing ink to page, please wait...", "Drawing ink into Notebook placeholder text");
    this.MESSAGE_EMPTY = NB.localize("Nothing on SMART kapp board yet", "kapp board is empty");
    // DOM elements
    this.elems = {
      $animatedButtonCont: $(".animated-button-cont"),
      $blackInk: $("#blackInk"),
      $blueInk: $("#blueInk"),
      $canvas: $("#canvas"),
      $canvasCont: $("#canvas-cont"),      
      $colorControls: $("#color-controls"),
      $colorButtons: $("#color-controls button"),
      $container: $("#container"),
      $controlsSidebarDivs: $("#controls-sidebar div"),
      $cropperBox: $("#cropper-box"),
      $cropClickable: $("#crop-clickable"),
      $curtainCont: $("#curtain-cont"),
      $curtainContDivs: $("#curtain-cont div"),
      $curtainTop: $("#curtain-top"),
      $curtainBottom: $("#curtain-bottom"),
      $curtainLeft: $("#curtain-left"),
      $curtainRight: $("#curtain-right"),
      $doc: $(document),
      $greenInk: $("#greenInk"),
      $help: $("help"),
      $helpCont: $("#help-cont"),
      $inkControls: $("#ink-controls"),
      $inkControlsImgs: $("#ink-controls img"),
      $inkInsertControls: $("#ink-insert-controls"),
      $inkCropControls: $("#ink-crop-controls"),      
      $insertInkClickable: $("#insertInk-clickable"),
      $insertInkNewPageClickable: $("#insertInkNewPage-clickable"),
      $liveStatus: $("#live-status"),
      $labeled: $(".labeled"),
      $loadingBar: $("#loading-bar"),
      $pageCover: $("#page-cover"),
      $pageDisable: $("#page-disable"),
      $playPause: $("#playPause"),
      $redInk: $("#redInk"),
      $scrubber: $("#scrubber"),
      $scrubberBackForward: $("#scrubber-back, #scrubber-forward"),
      $scrubberBackForwardImg: $("#scrubber-back img, #scrubber-forward img"),
      $scrubberCont: $("#scrubber-cont"),
      $scrubberControls: $("#scrubber-controls"),
      $scrubberControlsDivs: $("#scrubber-controls, #scrubber-controls div"),
      $skipBack: $("#skipBack"),
      $skipForward: $("#skipForward"),
      $statusBar: $("#status-bar"),
      $statusCont: $("#status-cont"),
      $stepBack: $("#stepBack"),
      $stepForward: $("#stepForward"),
      $tooltipCont: $("#tooltip-cont"),
      $tooltip: $("#tooltip"),
      $tooltipTitle: $("#tooltip-title"),
      $tooltipBody: $("#tooltip-body")
    };
  }

  UI.prototype.toggleCropping = function(state, activateLiveStatus, activateColorButtons) {
    if (state) {
      this.elems.$curtainCont.css("opacity", 0.4);
      this.elems.$cropperBox.css("opacity", 1);
    } else {
      this.elems.$curtainCont.css("opacity", 0);
      this.elems.$cropperBox.css("opacity", 0);
    }
    this.elems.$cropClickable.toggleClass("selected", state);
    if (state || activateLiveStatus) {
      this.status.toggleLive(!state);
    }
    if (state || activateColorButtons) {
      this.toggleColorButtons(!state);
    }
  };

  UI.prototype.toggleColorButtons = function(state) {
    var zInd = state ? 90 : -90;
    var opac = state ? 1 : 0;
    this.elems.$colorControls.css("z-index", zInd);
    this.elems.$colorControls.css("opacity", opac);
  };

  // Scales the canvas down to the window's dimensions.
  // Uses magic ratios to get the elements to scale to fit in the window.
  UI.prototype.initWindow = function() {
    // Total real estate
    var windowWidth = this.elems.$container.width();
    var windowHeight = this.elems.$container.height();
    // Board size
    var boardWidth = BoardState.session.boardWidth;
    var boardHeight = BoardState.session.boardHeight;
    // Reserve space for the right and bottom controls
    var width = 8/9 * windowWidth;
    var height = 9/10 * windowHeight;
    // Constrain the Canvas to either fit the width or the height
    var scale = boardWidth / boardHeight > width / height ? width / boardWidth : height / boardHeight;
    var canvasWidth = boardWidth * scale;
    var canvasHeight = boardHeight * scale;
    this.elems.$canvasCont.width(canvasWidth);
    this.elems.$canvasCont.height(canvasHeight);
    // Margins between the canvas and the edges of the screen
    var marginX = (windowWidth - canvasWidth) / 2; 
    var marginY = windowHeight - canvasHeight;
    // Center the canvas
    this.elems.$canvasCont.css("margin-left", marginX + "px");
    // Set the tooltip starting position
    this.elems.$tooltip.css("top", this.help.initialPosition.top());
    this.elems.$tooltip.css("left", this.help.initialPosition.left());
    this.elems.$tooltip.css("opacity", 0);

    //-- SideBar sizes ----------------------------------------------------
    var SB_buttonSize = marginX * 1/2; 
    var SB_iconSize = SB_buttonSize * 7/5; 
    var SB_fontSize = SB_buttonSize * 2/5;
    this.elems.$labeled.css("font-size", SB_fontSize+"px");
    this.elems.$inkControls.css("bottom",  windowHeight - canvasHeight + "px");
    this.elems.$inkControlsImgs.width(SB_iconSize).height(SB_iconSize);
    this.elems.$colorControls.css("top", 1/6 * canvasHeight + "px");
    this.elems.$colorButtons.width(SB_buttonSize).height(SB_buttonSize);
    this.elems.$help.width(SB_buttonSize).height(SB_buttonSize);
    this.elems.$helpCont.css("top", "0px");
    this.elems.$controlsSidebarDivs.width(marginX);
    
    //-- SCrubber sizes ---------------------------------------------------
    /* Control buttons for the scrubber */
    var SC_skipHeight = marginY * 1/3; //forward and backward button height, used as a reference size
    var SC_playSize = marginY * 1/2;
    var SC_skipOffset = (SC_playSize - SC_skipHeight) / 2;
    var SC_controlsBottom = (marginY - SC_skipHeight) / 2;
    this.elems.$scrubberControls.css("bottom", SC_controlsBottom + "px");
    this.elems.$scrubberBackForwardImg.height(SC_skipHeight);
    this.elems.$scrubberControlsDivs.height(SC_playSize);
    this.elems.$playPause.width(SC_playSize).height(SC_playSize);
    this.elems.$scrubberBackForward.css("margin-top", SC_skipOffset + "px");
    /* The scrubber itself */
    var SC_marginSum = 20; //From the CSS - sum of the margins between the edge of the window and the skip/step controls
    var SC_skipWidthSum = SC_marginSum + SC_skipHeight + SC_skipHeight*8/5; //Width of step button + skip button + all the margins
    var SC_scrubberTop = (SC_playSize - SC_skipHeight * 4/5) / 2;
    this.elems.$scrubberCont.css("margin-left", SC_playSize + SC_skipWidthSum + "px");
    this.elems.$scrubberCont.css("margin-right", SC_skipWidthSum + "px");
    this.elems.$scrubber.css("margin-top", SC_scrubberTop);
    var SC_scrubberTrack = SC_skipHeight * 4/5;
    var SC_scrubberThumb = SC_skipHeight * 5/4;
    var SC_scrubberThumbTop = SC_skipHeight * -1/5 ;
    var SC_scrubberCSS = "#scrubber::-webkit-slider-runnable-track {height: " + SC_scrubberTrack + "}\n" +
    "#scrubber::-webkit-slider-thumb {height: " + SC_scrubberThumb + 
        "px; width: " + SC_scrubberThumb + "px; margin-top: " + SC_scrubberThumbTop + "px;}\n";
    /* Snapshot ticks */
    var SC_tickWidth = SC_skipHeight * 3/5;
    var SC_tickHeight = SC_tickWidth * 3/2;
    var SC_tickTop = SC_skipHeight * 4/5;
    var SC_tickCSS = ".tick-mark {width: " + SC_tickWidth + "px; height: " + SC_tickHeight + "px; top: " + SC_tickTop + "px;}\n";
    /* Since tick marks are generated as we go, have to create a stylesheet change for them.
    Selecting the slider is hard in jquery due to selecting with '::', so we do it in a stylesheet. */
    var sheet = document.createElement('style');
    sheet.innerHTML = SC_tickCSS + SC_scrubberCSS;
    document.body.appendChild(sheet);
  };
  
  UI.prototype.buttonClickAnimation = function() {
    var $button = $(this).children(".animated-button");
    $button.toggleClass("selected", true);
    setTimeout(function(){ $button.toggleClass("selected", false); }, 150);
  };

  UI.prototype.inkButtonClick = function($clickedButton) {
    self.elems.$colorButtons.toggleClass("selected-ink", false);
    $clickedButton.toggleClass("selected-ink", true);
  };

/*
  ___ _____ _ _____ _   _ ___ 
 / __|_   _/_\_   _| | | / __|
 \__ \ | |/ _ \| | | |_| \__ \
 |___/ |_/_/ \_\_|  \___/|___/

*/                            
  
  UI.prototype.status = {
    toggleMessage: function(state, message, zInd) {
      if (state) {
        zInd = zInd ? zInd : 50;
        self.elems.$statusCont.css("opacity", 1);
        self.elems.$statusCont.css("z-index", zInd);
        self.elems.$statusBar[0].innerText = message;
      } else {
        self.elems.$statusCont.css("opacity", 0);
        self.elems.$statusCont.css("z-index", -100);
      }
    },

    loadingBarUpdate: function(fracCompleted) {
      var percent = fracCompleted * 100;
      self.elems.$loadingBar.css("width", percent + "%");
    },

    toggleLive: function(state) {
      var opac = state ? 1 : 0;
      self.elems.$liveStatus.css("opacity", opac);
    }
  };
/*
  _  _ ___ _    ___ 
 | || | __| |  | _ \
 | __ | _|| |__|  _/
 |_||_|___|____|_|  
                    
------------------------------------------------------------------------------------
Quick Help Guide
    UI.help.isHelp - when the help guide is open
    UI.help.initialPosition - the initial position of the tooltip
    UI.help.currentMessage - the current help message displayed
    UI.help.helpStep - which step of the help guide is being displayed
    UI.help.helpMessages - the array of help messages
    UI.help.statusOpacity - stores the previous opacity of the status bar to restore later
    UI.help.elementStyles - stores the previous styles of elements to restore later
------------------------------------------------------------------------------------
*/
  UI.prototype.help = {
    isHelp: false,
    currentMessage: undefined,
    statusOpacity: undefined,
    elementStyles: [],
    initialPosition: {
      top: function() { return self.elems.$helpCont.position().top - 5 + "px"; }, 
      left: function() { return self.elems.$helpCont.position().left - 300 + "px"; }
    },
    helpStep: 0,
    helpMessages: [
      { title: NB.localize("Quick Start Tour (%d/%d)", "help tour title").replace("%d", "1").replace("%d", "6"),
        body: NB.localize("Tap anywhere to proceed. <br/> Tap the Help button to stop at any time.", "help text"),
        top: function() { return self.elems.$helpCont.position().top - 5 + "px"; },
        left: function() { return self.elems.$helpCont.position().left - 265 + "px"; },
      },
      { title: NB.localize("SMART kapp Board View (%d/%d)", "help tour title").replace("%d", "2").replace("%d", "6"),
        body: NB.localize("This is a view of what's written on the<br/>SMART kapp board.", "help text"),
        top: function() { return "100px"; },
        left: function() { return (self.elems.$container.outerWidth() - 292) / 2 + "px"; },
        elements: function() { return self.elems.$canvas; }
      },
      { title: NB.localize("Live Status (%d/%d)", "help tour title").replace("%d", "3").replace("%d", "6"),
        body: NB.localize("The Board View is 'Live' when you are connected,<br/> and the timeline is all the way to the right.", "help text"),
        top: function() { return self.elems.$liveStatus.position().top - 15 + "px"; },
        left: function() { return self.elems.$liveStatus.position().left + 115 + "px"; },
        elements: function() { return self.elems.$liveStatus; },
        effects: function() { self.elems.$liveStatus.css("opacity", 1); }
      },
      { title: NB.localize("Timeline (%d/%d)", "help tour title").replace("%d", "4").replace("%d", "6"),
        body: NB.localize("Use these controls to view the board's history.<br/>Tap or hold the step buttons to rewind or play forward.<br/>Tap the skip buttons to jump to important points.", "help text"),
        top: function() { return self.elems.$scrubberControls.position().top - 100 + "px"; },
        left: function() { return (self.elems.$container.outerWidth() - 403) / 2 + "px"; },
        elements: function() { return self.elems.$scrubberControls; },
        effects: function() { self.elems.$scrubberControls.css("background-color", "white"); }
      },
      {
        title: NB.localize("Drawing into SMART Notebook (%d/%d)", "help tour title").replace("%d", "5").replace("%d", "6"),
        body: NB.localize("The ink buttons draw your SMART kapp board as<br/>Notebook ink onto either the current page or a<br/>new page.", "help text"),
        top: function() { return self.elems.$inkControls.position().top + self.elems.$inkInsertControls.position().top + 30 + "px"; },
        left: function() { return self.elems.$inkControls.position().left + self.elems.$inkInsertControls.position().left - 310 + "px"; },
        elements: function() { return $("#ink-controls, #ink-insert-controls"); },
        effects: function() { self.elems.$inkInsertControls.css("background-color", "white"); }
      },
      { title: NB.localize("Cropper Tool (%d/%d)", "help tour title").replace("%d", "6").replace("%d", "6"),
        body: NB.localize("Tap the 'Crop' button to enter cropping mode.<br/>When you have an area of the board selected<br/>with the cropper, the ink buttons will draw<br/>only that selection to the page.", "help text"),
        top: function() { return self.elems.$inkControls.position().top + self.elems.$inkInsertControls.position().top - 50 + "px"; },
        left: function() { return self.elems.$inkControls.position().left + self.elems.$inkInsertControls.position().left - 290 + "px"; },
        elements: function() { return $("#canvas, #curtain-cont, #curtain-top, #curtain-bottom, #curtain-left, #curtain-right, #cropper-box, #ink-controls, #ink-crop-controls"); },
        effects: function() { self.elems.$inkCropControls.css("background-color", "white"); self.crop.showDemoCropbox(); },
      }//,
      // { title: NB.localize("SMART kapp Colors (%d/%d)", "help tour title").replace("%d", "7").replace("%d", "7"),
      //   body: NB.localize("Color controls appear when the Board View is<br/>'Live', and represent the virtual color of your<br/>whiteboard marker. Affects ONLY future<br/>strokes from the SMART kapp board.", "help text"),
      //   top: function() { return self.elems.$colorControls.position().top + 35 + "px"; },
      //   left: function() { return self.elems.$colorControls.position().left - 290 + "px"; },
      //   elements: function() { return self.elems.$colorControls; },
      //   effects: function() { self.elems.$colorControls.css({"background-color": "white", "opacity": 1}); }
      // }
    ],
    toggleHelp: function(state) {
      if (state) {
        //Since it has a css transition when the opacity is changed, convert to 1/0
        this.statusOpacity = self.elems.$statusCont.css("opacity") == 1 ? 1 : 0;
        self.elems.$statusCont.css("opacity", 0);
        self.elems.$pageCover.css("opacity", 0.4);
        self.elems.$pageDisable.css("z-index", 200);
        self.elems.$pageCover.css("z-index", 100);
        self.elems.$tooltipCont.css("z-index", 1000);
        self.elems.$helpCont.toggleClass("selected", true);
        this.isHelp = true;
        this.helpStep = 0;
      } else {
        self.elems.$statusCont.css("opacity", this.statusOpacity);
        self.elems.$pageCover.css("opacity", 0);
        self.elems.$pageDisable.css("z-index", -200);
        self.elems.$pageCover.css("z-index", -100);
        self.elems.$tooltipCont.css("z-index", -1000);
        self.elems.$helpCont.toggleClass("selected", false);
        self.elems.$tooltip.css("top", this.initialPosition.top());
        self.elems.$tooltip.css("left", this.initialPosition.left());
        self.elems.$tooltip.css("opacity", 0);
        this.isHelp = false;
        this.resetPreviousStep();
      }
    },

    // Advances to the next step in the tutorial
    advanceHelp: function() {
      if (this.helpStep === this.helpMessages.length) { 
        this.toggleHelp(false); 
        return; 
      }
      var message = this.helpMessages[this.helpStep];
      self.elems.$tooltipTitle[0].innerHTML = message.title;
      self.elems.$tooltipBody[0].innerHTML = message.body;
      self.elems.$tooltip.css("top", message.top());
      self.elems.$tooltip.css("left", message.left());
      self.elems.$tooltip.css("opacity", 1);
      if (message.elements) {
        this.activateElements(message.elements());
      }
      if (message.effects) {
        message.effects();
      }
      this.helpStep++;
      this.currentMessage = message;
    },

    // Brings the elements in front of the cover, but beneath the disabler
    // Makes it so people can look, but not touch
    activateElements: function(elements) {
      var elementsLen = elements.length;
      for (var i = 0; i < elementsLen; i++) {
        this.elementStyles.push(elements[i].style.cssText);
        $(elements[i]).css("z-index", 150);
      }
    },

    resetPreviousStep: function() {
      if (this.currentMessage) {
        if (this.currentMessage.elements) {
          this.deactivateElements(this.currentMessage.elements());
        }
        if (this.currentMessage.reset) {
          this.currentMessage.reset();
        }
        this.currentMessage = undefined;
      }
    },

    // Reverts the elements' css back to what they were, should be match 1-to-1 with activate
    deactivateElements: function(elements) {
      var elementsLen = elements.length;
      for (var i = 0; i < elementsLen; i++) {
        elements[i].style.cssText = this.elementStyles[i];
      }
      this.elementStyles = [];
    },

    advanceHelpIfActive: function() {
      if (self.help.isHelp) {
        self.help.resetPreviousStep();
        self.help.advanceHelp();
      }
    }
  };
  UI.prototype.crop = {
    // Draws what looks the same as a normal cropping window - for visual demonstration ONLY
    showDemoCropbox: function() {
      self.elems.$curtainCont.css("opacity", 0.4);
      self.elems.$cropperBox.css("opacity", 1);
      this.drawCropperWindow({x: 100, y: 50}, {x: 400, y: 250});
    },

    hideDemoCropbox: function() {
      this.elems.$curtainCont.css("opacity", 0);
      this.elems.$cropperBox.css("opacity", 0);
      this.drawCropperWindow({x:0, y:0}, {x:0, y:0});
    },

    // Redraws the curtain and cropper box based on two points representing the box
    drawCropperWindow: function(startPoint, endPoint) {
      var start = {};
      var end = {};
      var curtain = self.elems.$curtainCont;
      var box = self.elems.$cropperBox;
      var top = self.elems.$curtainTop;
      var bottom = self.elems.$curtainBottom;
      var left = self.elems.$curtainLeft;
      var right = self.elems.$curtainRight;
      // Check the bounds of the box
      if (startPoint.x < 0) {
        startPoint.x = 0;
      } else if (startPoint.x > curtain.width()) {
        startPoint.x = curtain.width();
      }
      if (startPoint.y < 0) {
        startPoint.y = 0;
      } else if (startPoint.y > curtain.height()) {
        startPoint.y = curtain.height();
      }
      if (endPoint.x < 0) {
        endPoint.x = 0;
      } else if (endPoint.x > curtain.width()) {
        endPoint.x = curtain.width();
      }
      if (endPoint.y < 0) {
        endPoint.y = 0;
      } else if (endPoint.y > curtain.height()) {
        endPoint.y = curtain.height();
      }
      // Make sure Start is always less than End
      start.x = startPoint.x > endPoint.x ? endPoint.x : startPoint.x;
      start.y = startPoint.y > endPoint.y ? endPoint.y : startPoint.y;
      end.x = startPoint.x > endPoint.x ? startPoint.x : endPoint.x;
      end.y = startPoint.y > endPoint.y ? startPoint.y : endPoint.y;
      top.height(start.y);
      bottom.height(curtain.height() - end.y);
      left.width(start.x);
      right.width(curtain.width() - end.x);
      // Offsets used to get the dashed lines within the shaded region
      var offset = 0;
      box.css('top', start.y + offset + 'px');
      box.css('left', start.x + offset + 'px');
      box.width(end.x - start.x - 4);
      box.height(end.y - start.y - 4);
    }
  };
  return UI;
}();
