// Handles updating and managing the view(s). Connects the view(s) and SessionHandler

var ViewHandler = function() {

	var sessionHandler;
	var observers;

	function ViewHandler(newSessionHandler) {
		sessionHandler = newSessionHandler;
		observers = {};
	}

	// Connects if disconnected, and disconnects if connected.
	// Meant to be called from views that don't know the current
	// connection state.
	ViewHandler.prototype.toggleConnection = function (sessionid) {
		if (checkSessionStatus("connected")) {
		    this.disconnect();
		} else if(!checkSessionStatus("connecting")) {
			this.connect(sessionid);
		}
	};

	// Connects to the board
	ViewHandler.prototype.connect = function (sessionid) {
	    if (checkSessionStatus("error")) {
	    	updateViews("disconnected");
	    }
	    trackEvent('button', 'click', 'connect', 1);
	    if (checkValidID(sessionid)) {
	    	sessionHandler.connect(sessionid, updateViews);
	    } else {
	    	updateViews("invalidSessionID");
	    	trackEvent('backend', 'connection', 'invalid sessionid (< 8-chars)', 1);
	    }
	};

	// Disconnects from the board
	ViewHandler.prototype.disconnect = function () {
		trackEvent('button', 'click', 'disconnect', 1);
		sessionHandler.disconnect(updateViews);
	};

	// Opens the dialog
	ViewHandler.prototype.openDialog = function () {
		trackEvent('button', 'click', 'dialog button', 1);
		sessionHandler.openDialog(updateViews);
	};

	// Adds an observer, to be called upon a change from the model
	// Implemented events: connected, connecting, error, disconnected
	// dialogOpened, dialogClosed, invalidSessionID
	ViewHandler.prototype.addObserver = function(event, callback) {
		if(!observers[event]) {
			observers[event] = [];
		}
		observers[event].push(callback);
	};

	// Notifies all views of the given event, calling their given
	// callbacks with the passed in data
	function updateViews(event, data) {
		var callbacks = observers[event];
		for(var i = 0; i < callbacks.length; i++) {
			callbacks[i](data);
		}
	}

	// Checks if the given status is correct
	function checkSessionStatus(status) {
		return sessionHandler.checkSessionStatus(status);
	}

	// Make sure ID is 8 characters
	function checkValidID(sessionid) {
		return sessionid.length == 8;
	}

	return ViewHandler;

}();