// The sidebar view of the kapp addon

var View = function() {

	var viewHandler;

	var idField;
	var statusRing;
	var connectionButton;
	var playbackButton;
	var playbackContainer;
	var tooltip;
	var toolTipContainer;
	var allControls;

	var tooltipText = {
		"enterId" : NB.localize("Enter your sharing ID, the last 8-characters found in the sharing URL from the mobile app.", "Tooltip on how to enter a kapp session id"),
		"connected" : NB.localize("Connected to SMART kapp board. Tap Open Board View to see your session.", "Tooltip for how to view the contents of a kapp session."),
		"error" : {
			"sessionFull" : NB.localize("Session is full. Purchase a subscription to enable more simultaneous connections.", "Error message when a kapp session is full."),
			"noSession" : NB.localize("Could not join the requested session. Try again with a valid sharing ID.", "Could not join the requested session. Try again with a valid sharing ID."),
			"withCode" : NB.localize("Unknown connection error with code %s. Please try again.", "Connection error message")
		}
	};

	View.prototype.translate = function() {
		$("#id").attr("placeholder", NB.localize("Enter ID", "placeholder text for enter a session ID text field"));
	}

	function View(newViewHandler) {
		viewHandler = newViewHandler;
		addObservers();

		idField = $("#id");
		statusRing = $("#status-ring");
		connectionButton = $("#connect");
		playbackButton = $("#playback");
		playbackContainer = $("#playback-cont");
		tooltip = $("#tooltip")[0];
		toolTipContainer = $("#tooltip-cont");
		allControls = $("#session-cont, #id");
		
		// Connect/disconnect button
		connectionButton.click(function () {
			var sessionid = getConnectionID();
			viewHandler.toggleConnection(sessionid);
		});

		// Binds [RETURN] in the ID box to trigger the connect/disconnect button
		idField.keyup(function (event) {
			if(event.keyCode == 13) {
				connectionButton.click();
			}
		});

		// Session ID Validation filter - only allow lowercase alphanumeric
		idField.on('input', function() {
			var input = $(this).val();
			var validInput = input.toLowerCase().replace(/[^a-z0-9]/g, '');
			if (validInput != input) $(this).val(validInput);
		});

		// Brings up the current tooltip upon clicking help
		statusRing.click(function (event) {
			displayTooltip(event);
		});

		// Button to open dialog window
		playbackButton.click(function() {
			viewHandler.openDialog();
		});

		$(document).click(function() {
			toolTipContainer.css("visibility", "hidden");
		});

		showDisconnected();

	}

	function showConnected() {
		clearStatusRingAnimation();
	    setStatusRing("connected");
	    connectionButton[0].innerText = NB.localize("Disconnect", "Disconnect from a kapp session");
	    allControls.toggleClass("disabled", true);
	    allControls.toggleClass("unselectable", true);
	    idField.prop('disabled', true);
	    tooltip.innerText = tooltipText.connected;
	    insertPlaybackButton();
	}

	function showConnecting() {
	    clearStatusRingAnimation();
	    setStatusRing("connecting");
	    connectionButton[0].innerText = NB.localize("Connecting...", "Connecting to a kapp session placeholder text.");
	    allControls.toggleClass("unselectable", true);
	    idField.prop('disabled', true);
	    removePlaybackButton();
	}

	function showDisconnected() {
		clearStatusRingAnimation();
		setStatusRing("disconnected");
		connectionButton[0].innerText = NB.localize("Connect", "Connect to a kapp session.");
		allControls.toggleClass("disabled", false);
		allControls.toggleClass("unselectable", false);
		idField.prop('disabled', false);
		tooltip.innerText = tooltipText.enterId;
	}

	function showError(errcode) {
		clearStatusRingAnimation();
		setStatusRing("error");
		connectionButton[0].innerText = NB.localize("Connect", "Connect to a kapp session.");
		allControls.toggleClass("disabled", false);
		allControls.toggleClass("unselectable", false);
		idField.prop('disabled', false);
		switch(errcode) {
		case 403:
			tooltip.innerText = tooltipText.error.sessionFull;
			printDebug("Connection Error: Session is full.");
			break;
		case 404:
			tooltip.innerText = tooltipText.error.noSession;
			printDebug("Connection Error: Session not found.");
			break;
		default:
			tooltip.innerText = tooltipText.error.withCode.replace("%s, errcode");
			printDebug("Connection Error: Unknown.");
			break;
		}
		displayTooltip();
	}

	// Sets the current state of the status ring
	function setStatusRing(state) {
	  if (state === "connected") {
		statusRing.toggleClass("error", false);
		statusRing.toggleClass("disconnected", false);
		statusRing.toggleClass("connected", true); 
	  } else if (state === "disconnected") {
	    statusRing.toggleClass("error", false);
	    statusRing.toggleClass("connected", false); 
	    statusRing.toggleClass("disconnected", true);
	  } else if (state === "error") {
	    statusRing.toggleClass("connected", false); 
	    statusRing.toggleClass("disconnected", false);
	    statusRing.toggleClass("error", true);
	  } else if (state === "connecting") {
	    animateConnecting();
	  }
	}

	var statusRingAnimationInterval;
	// Starts the pulsing green animation for connecting
	function animateConnecting() {
	  if (statusRing.hasClass("disconnected")) {
	    setStatusRing("connected");
	  } else {
	    setStatusRing("disconnected");
	  }
	  statusRingAnimationInterval = setTimeout(animateConnecting, 1000);
	}

	// Clears the current animation
	function clearStatusRingAnimation() {
	  clearTimeout(statusRingAnimationInterval);
	}

	// Insert the button to open the dialog
	function insertPlaybackButton() {
	  playbackContainer.css("opacity", 1);
	  setTimeout(function() {
	    playbackContainer.css("pointer-events", "auto");
	  }, 200);
	}

	// Remove the button to open the dialog
	function removePlaybackButton() {
	  playbackContainer.css("opacity", 0);
	  playbackContainer.css("pointer-events", "none");
	}

	// Toggles on the visibility of the tooltip
	function displayTooltip(event) {
		setTimeout(function() {
			toolTipContainer.css("visibility", "visible");
		}, 100);
	}

	// Get the connection ID entered by the user in the box
	function getConnectionID() {
	  return idField.val();
	}

	function addObservers() {
		viewHandler.addObserver("connected", showConnected);
		viewHandler.addObserver("connecting", showConnecting);
		viewHandler.addObserver("error", showError);
		viewHandler.addObserver("disconnected", showDisconnected);
		viewHandler.addObserver("dialogOpened", removePlaybackButton);
		viewHandler.addObserver("dialogClosed", insertPlaybackButton);
		viewHandler.addObserver("invalidSessionID", displayTooltip);
	}

	return View;

}();

